require 'thor'

module Slipsquare
  autoload :Middleware, "slipsquare/middleware"

  class CLI < Thor
    include Thor::Actions
    ENV['THOR_COLUMNS'] = '120'

    !check_unknown_options

    map "--version"      => :version,
        "-v"             => :version

    desc "help [COMMAND]", "Describe commands or a specific command"
    def help(meth=nil)
      super
      if !meth
        say "To learn more or to contribute, please see github.com/petems/slipsquare"
      end
    end

    desc "authorize", "Authorize a Dropbox account with slipsquare"
    long_desc "This takes you through a workflow for adding configuration
    details to slipsquare."
    def authorize
      Middleware.sequence_authorize.call({})
    end


    desc "account", "Shows Dropbox account details"
    long_desc "This displays your main Dropbox meta-data, such as
    Display Name, County, Download Quota information and the like"
    def account
      Middleware.sequence_account.call({})
    end

    desc "download [FILENAME]", "Download a file or directory"
    long_desc "Download or directory from dropbox"
    def download(filename)
      Middleware.sequence_download_file.call({"download_file_name" => filename})
    end

    desc "verify", "Check your given credentials are valid"
    long_desc "This checks that the credentials in ~/.slipsquare are correct"
    def verify
      Middleware.sequence_verify.call({})
    end

    desc "ls", "Show files for a particular path"
    method_option "path",
                  :type => :string,
                  :aliases => "-p",
                  :desc => "The path you want to give."
    long_desc "This shows all the files of a given path (defaulting to root)"
    def ls(path=nil)
      Middleware.sequence_ls.call({"list_files_path" => path})
    end

    desc "mkdir [PATH]", "Make a directory on dropbox"
    long_desc "This creates a directory on Dropbox, it allows you to enter
    multiple directories and all will be created eg. `slipsquare mkdir photos/america/san_fran"
    def mkdir(path)
      Middleware.sequence_mkdir.call({"mkdir_path" => path})
    end

    desc "delete [PATH]", "Delete a directory or file on Dropbox"
    long_desc "This creates a directory on Dropbox"
    def delete(path)
      Middleware.sequence_delete_file.call({"delete_path" => path})
    end

    desc "upload [PATH]", "Upload a file or directory"
    long_desc "Upload a file or directory to dropbox"
    def upload(filename)
      Middleware.sequence_upload_file.call({"upload_file_name" => filename})
    end

    desc "chunked-upload [PATH]", "Chunk Upload a file or directory"
    long_desc "Upload a file or directory to dropbox with chunks, typically
    4mb in size, allowing a way of showing progress of downloads."
    def chunked_upload(filename)
      Middleware.sequence_chunked_upload_file.call({"chunked_upload_file_name" => filename})
    end

    desc "get-keys", "Get your app and secret keys"
    def get_keys
      say "Please open this URL from your Browser, and login: https://www2.dropbox.com/developers/apps"
      say "Click \"Create app\" button, and choose the following options:"
      say "Type: Dropbox API app"
      say "Data: Files and Datastores"
      say "Permission type: Full Dropbox"
      say "File Types: All file types"
      say "App name: Slipsquare#{Time.now.to_i}"
      say "Cick on the \"Create\" button, and note down the 'app key' and 'app secret'"
    end

    desc "version", "Show version"
    def version
      say "Slipsquare #{Slipsquare::VERSION}"
    end

  end
end

