require "middleware"

module Slipsquare
  module Middleware
    autoload :Base, "slipsquare/middleware/base"

    autoload :AskForCredentials, "slipsquare/middleware/ask_for_credentials"
    autoload :Account, "slipsquare/middleware/account"
    autoload :CheckConfiguration, "slipsquare/middleware/check_configuration"
    autoload :CheckCredentials, "slipsquare/middleware/check_credentials"
    autoload :ChunkedUpload, "slipsquare/middleware/chunked_upload"
    autoload :DeleteFile, "slipsquare/middleware/delete_file"
    autoload :DownloadFile, "slipsquare/middleware/download_file"
    autoload :InjectClient, "slipsquare/middleware/inject_client"
    autoload :InjectConfiguration, "slipsquare/middleware/inject_configuration"
    autoload :ListFiles, "slipsquare/middleware/list_files"
    autoload :MakeDirectory, "slipsquare/middleware/make_directory"
    autoload :UploadFile, "slipsquare/middleware/upload_file"

    # Start the authorization flow.
    # This writes a ~/.slipsquare file, which can be edited manually.
    def self.sequence_authorize
      ::Middleware::Builder.new do
        use InjectConfiguration
        use AskForCredentials
        use InjectConfiguration
        use CheckConfiguration
        use InjectClient
        use CheckCredentials
      end
    end

    def self.sequence_account
      ::Middleware::Builder.new do
        use InjectConfiguration
        use CheckConfiguration
        use InjectClient
        use Account
      end
    end

    def self.sequence_verify
      ::Middleware::Builder.new do
        use InjectConfiguration
        use CheckConfiguration
        use InjectClient
        use CheckCredentials
      end
    end

    def self.sequence_ls
      ::Middleware::Builder.new do
        use InjectConfiguration
        use CheckConfiguration
        use InjectClient
        use ListFiles
      end
    end

    def self.sequence_mkdir
      ::Middleware::Builder.new do
        use InjectConfiguration
        use CheckConfiguration
        use InjectClient
        use MakeDirectory
      end
    end

    def self.sequence_upload_file
      ::Middleware::Builder.new do
        use InjectConfiguration
        use CheckConfiguration
        use InjectClient
        use UploadFile
      end
    end

    def self.sequence_download_file
      ::Middleware::Builder.new do
        use InjectConfiguration
        use CheckConfiguration
        use InjectClient
        use DownloadFile
      end
    end

    def self.sequence_delete_file
      ::Middleware::Builder.new do
        use InjectConfiguration
        use CheckConfiguration
        use InjectClient
        use DeleteFile
      end
    end

    def self.sequence_chunked_upload_file
      ::Middleware::Builder.new do
        use InjectConfiguration
        use CheckConfiguration
        use InjectClient
        use ChunkedUpload
      end
    end

  end
end
